using System;
using System.Collections.Generic;
using System.Text;

namespace XnaDevRu.Bullet
{
	/// <summary>
	/// CollisionAlgorithm is an collision interface that is compatible with the Broadphase and Dispatcher.
	/// It is persistent over frames
	/// </summary>
	public abstract class CollisionAlgorithm
	{
		private IDispatcher _dispatcher;

		public CollisionAlgorithm(CollisionAlgorithmConstructionInfo collisionAlgorithmConstructionInfo)
		{
			_dispatcher = collisionAlgorithmConstructionInfo.Dispatcher;
		}

		protected IDispatcher Dispatcher { get { return _dispatcher; } set { _dispatcher = value; } }

		public abstract void ProcessCollision(CollisionObject colA, CollisionObject colB, DispatcherInfo dispatchInfo, ManifoldResult resultOut);
		public abstract float CalculateTimeOfImpact(CollisionObject colA, CollisionObject colB, DispatcherInfo dispatchInfo, ManifoldResult resultOut);
	}
}
