using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;

namespace XnaDevRu.Bullet
{
	public class CylinderShapeX : CylinderShape
	{
		public CylinderShapeX(Vector3 halfExtents)
			: base(halfExtents) { }

		public override int UpAxis
		{
			get
			{
				return 0;
			}
		}

		public override float Radius
		{
			get
			{
				return HalfExtents.Y;
			}
		}

		//debugging
		public override string Name
		{
			get
			{
				return "CylinderX";
			}
		}

		public override Vector3 LocalGetSupportingVertexWithoutMargin(Vector3 vec)
		{
			return CylinderLocalSupportX(HalfExtents, vec);
		}

		public override void BatchedUnitVectorGetSupportingVertexWithoutMargin(Vector3[] vectors, Vector3[] supportVerticesOut)
		{
			for (int i = 0; i < vectors.Length; i++)
			{
				supportVerticesOut[i] = CylinderLocalSupportX(HalfExtents, vectors[i]);
			}
		}

		private Vector3 CylinderLocalSupportX(Vector3 halfExtents, Vector3 v)
		{
			//mapping depends on how cylinder local orientation is
			// extents of the cylinder is: X,Y is for radius, and Z for height
			float radius = halfExtents.Y;
			float halfHeight = halfExtents.X;

			Vector3 tmp = new Vector3();
			float d;

			float s = (float)Math.Sqrt(v.Y * v.Y + v.Z * v.Z);
			if (s != 0)
			{
				d = radius / s;
				tmp.Y = v.Y * d;
				tmp.X = v.X < 0 ? -halfHeight : halfHeight;
				tmp.Z = v.Z * d;
				return tmp;
			}
			else
			{
				tmp.Y = radius;
				tmp.X = v.X < 0 ? -halfHeight : halfHeight;
				tmp.Z = 0;
				return tmp;
			}
		}
	}
}
